@@ -164,6 +164,8 @@ get_rdp_filetime(time_t seconds)
 static time_t
 get_system_filetime(uint64 rdp_time)
 {
+	if (rdp_time == 0LL || rdp_time == (uint64)(-1LL))
+		return 0;
 	return (time_t) (rdp_time / 10000000LL - 11644473600LL);
 }
 
@@ -549,7 +551,7 @@ disk_query_volume_info(IRP * irp)
 			memset(buf, 0, 256);
 			SET_UINT64(buf, 0, 0); /* VolumeCreationTime */
 			SET_UINT32(buf, 8, 0); /* VolumeSerialNumber */
-			len = set_wstr(buf + 17, size - 17, "FREERDP", strlen("FREERDP") + 1);
+			len = freerdp_set_wstr(buf + 17, size - 17, "FREERDP", strlen("FREERDP") + 1);
 			SET_UINT32(buf, 12, len); /* VolumeLabelLength */
 			SET_UINT8(buf, 16, 0);	/* SupportsObjects */
 			size = 17 + len;
@@ -570,7 +572,7 @@ disk_query_volume_info(IRP * irp)
 			memset(buf, 0, 256);
 			SET_UINT32(buf, 0, FILE_CASE_SENSITIVE_SEARCH | FILE_CASE_PRESERVED_NAMES | FILE_UNICODE_ON_DISK); /* FileSystemAttributes */
 			SET_UINT32(buf, 4, F_NAMELEN(stat_fs)); /* MaximumComponentNameLength */
-			len = set_wstr(buf + 12, 256 - 12, "FREERDP", 8);
+			len = freerdp_set_wstr(buf + 12, 256 - 12, "FREERDP", 8);
 			SET_UINT32(buf, 8, len); /* FileSystemNameLength */
 			size = 12 + len;
 			break;
@@ -681,6 +683,7 @@ disk_set_info(IRP * irp)
 	struct utimbuf tvs;
 	int mode;
 	uint32 attr;
+	time_t t;
 
 	LLOGLN(10, ("disk_set_info: class=%d id=%d", irp->infoClass, irp->fileID));
 	finfo = disk_get_file_info(irp->dev, irp->fileID);
@@ -695,17 +698,24 @@ disk_set_info(IRP * irp)
 	switch (irp->infoClass)
 	{
 		case FileBasicInformation:
+			if (stat(finfo->fullpath, &file_stat) != 0)
+				return get_error_status();
+
 			/* Change file time */
-			tvs.actime = get_system_filetime(GET_UINT64(irp->inputBuffer, 8)); /* LastAccessTime */
-			tvs.modtime = get_system_filetime(GET_UINT64(irp->inputBuffer, 16)); /* LastWriteTime */
+			tvs.actime = file_stat.st_atime;
+			tvs.modtime = file_stat.st_mtime;
+			t = get_system_filetime(GET_UINT64(irp->inputBuffer, 8)); /* LastAccessTime */
+			if (t > 0)
+				tvs.actime = t;
+			t = get_system_filetime(GET_UINT64(irp->inputBuffer, 16)); /* LastWriteTime */
+			if (t > 0)
+				tvs.modtime = t;
 			utime(finfo->fullpath, &tvs);
 
 			/* Change read-only flag */
 			attr = GET_UINT32(irp->inputBuffer, 32);
 			if (attr == 0)
 				break;
-			if (stat(finfo->fullpath, &file_stat) != 0)
-				return get_error_status();
 			mode = file_stat.st_mode;
 			if (attr & FILE_ATTRIBUTE_READONLY)
 				mode &= ~(S_IWUSR | S_IWGRP | S_IWOTH);
@@ -733,7 +743,7 @@ disk_set_info(IRP * irp)
 			size = len * 2;
 			buf = malloc(size);
 			memset(buf, 0, size);
-			get_wstr(buf, size, irp->inputBuffer + 6, len);
+			freerdp_get_wstr(buf, size, irp->inputBuffer + 6, len);
 			fullpath = disk_get_fullpath(irp->dev, buf);
 			free(buf);
 			LLOGLN(10, ("disk_set_info: rename %s to %s", finfo->fullpath, fullpath));
@@ -838,7 +848,7 @@ disk_query_directory(IRP * irp, uint8 initialQuery, const char * path)
 			/* [MS-FSCC] has one byte padding here but RDP does not! */
 			//SET_UINT8(buf, 69, 0); /* Reserved */
 			/* ShortName 24  bytes */
-			len = set_wstr(buf + 93, size - 93, pdirent->d_name, strlen(pdirent->d_name));
+			len = freerdp_set_wstr(buf + 93, size - 93, pdirent->d_name, strlen(pdirent->d_name));
 			SET_UINT32(buf, 60, len); /* FileNameLength */
 			size = 93 + len;
 			break;
@@ -859,7 +869,7 @@ disk_query_directory(IRP * irp, uint8 initialQuery, const char * path)
 			SET_UINT64(buf, 48, file_stat.st_size); /* AllocationSize */
 			SET_UINT32(buf, 56, attr); /* FileAttributes */
 			SET_UINT32(buf, 64, 0); /* EaSize */
-			len = set_wstr(buf + 68, size - 68, pdirent->d_name, strlen(pdirent->d_name));
+			len = freerdp_set_wstr(buf + 68, size - 68, pdirent->d_name, strlen(pdirent->d_name));
 			SET_UINT32(buf, 60, len); /* FileNameLength */
 			size = 68 + len;
 			break;
